@@ -48,7 +48,7 @@ class VirtualDirectory(object):
             pass
 
 class RecentDocumentsDirectory(VirtualDirectory):
-    def __init__(self, maxitems=10):
+    def __init__(self, maxitems=200):
         VirtualDirectory.__init__(self, 'recent')
 
         self._maxitems = maxitems
@@ -58,7 +58,7 @@ class RecentDocumentsDirectory(VirtualDirectory):
         manager = Gtk.RecentManager.get_default()
 
         items = manager.get_items()
-        items.sort(key=lambda a: a.get_visited())
+        items.sort(key=lambda a: a.get_visited(), reverse=True)
 
         added = 0
 
